import logging
from a1_loggermanager import my_logger, my_formatter
from a1_loggermanager.handlers import my_file_handler, my_rotating_file_handler, my_stream_handler, my_timed_rotating_file_handler

class LoggerManager:

  my_formatters: dict[str:]
  my_handlers: dict[str:]
  my_loggers: dict[str:]

  DEFAULT_HANDLER_TYPE = 'StreamHandler'

  logger: my_logger.MyLogger
  
  def __init__(self, parms = {}):
    #
    self.my_formatters = {}
    self.my_handlers = {}
    self.my_loggers = {}

    # add default formatter, handler and logger
    self.my_formatters['default'] = self.set_default_my_formatter()
    self.my_handlers['default'] = self.set_default_my_handler()
    self.my_loggers['default'] = self.set_default_my_logger()

    # set logger
    self.logger = self.my_loggers['default'].logger      

    # add values passed as parms
    formatters = parms.get('formatters')
    if formatters:
      self.add_my_formatters(formatters)
    handlers = parms.get('handlers')
    if handlers:
      self.add_my_handlers(handlers)
    loggers = parms.get('loggers')
    if loggers:
      self.add_my_loggers(loggers)

  def add_my_formatter(self, my_formattr):
    self.my_formatters[my_formattr['name']] = my_formatter.MyFormatter(my_formattr['format'], my_formattr.get('parms', {}))

  def add_my_formatters(self, my_formatters):
    for my_formatter in my_formatters:
      self.add_my_formatter(my_formatter)

  
  
  def add_my_handler(self, my_handler):
  #def add_my_handler(self, my_handler, filepath = None):

    #filepath = my_handler['parms']['filepath'] if filepath is None else filepath

    try:
      #handler_type = my_handler.get('handler_type', 'StreamHandler')  
      handler_type = my_handler.get('handler_type', self.DEFAULT_HANDLER_TYPE)  
      match handler_type:
        case 'FileHandler':

          self.my_handlers[my_handler['name']] = my_file_handler.MyFileHandler(my_handler['name'], my_handler['parms']['filepath'], my_handler.get('parms', {}))
          #self.my_handlers[my_handler['name']] = my_file_handler.MyFileHandler(my_handler['name'], filepath, my_handler.get('parms', {}))


        case 'RotatingFileHandler':
          self.my_handlers[my_handler['name']] = my_rotating_file_handler.MyRotatingFileHandler(my_handler['name'], my_handler['parms']['filepath'], my_handler.get('parms', {}))
        case 'StreamHandler':
          self.my_handlers[my_handler['name']] = my_stream_handler.MyStreamHandler(my_handler['name'])
        case 'TimedRotatingFileHandler':
          self.my_handlers[my_handler['name']] = my_timed_rotating_file_handler.MyTimedRotatingFileHandler(my_handler['name'], my_handler['parms']['filepath'], my_handler.get('parms', {}))
        case _:
          self.logger.error(f"Unknown handler type '{handler_type}'")
      # add formatter to handler if specified otherwise use default formatter
      self.my_handlers[my_handler['name']].set_formatter(self.my_formatters[my_handler.get('formatter_name', "default")].formatter)
    except:
      mess = f"Could not add handler with {my_handler}"
      self.logger.error(mess)
      raise Exception(mess)

  def add_my_handlers(self, my_handlers):
    for my_handler in my_handlers:
      self.add_my_handler(my_handler)

  def add_my_logger(self, my_loggr):
    name = my_loggr.get('name', 'default')
    self.my_loggers[name] = my_logger.MyLogger(name, my_loggr.get('level', None))
        # add default handler if none specified
    if not my_loggr.get('handler_names'):
      my_loggr['handler_names'] = ('default',)
    # add handlers to loggers
    for my_handler_name in my_loggr['handler_names']:
        self.my_loggers[name].add_handler(self.my_handlers[my_handler_name].handler)
    return self.my_loggers[name]        

  def add_my_loggers(self, my_loggers):
    for my_logger in my_loggers:
      self.add_my_logger(my_logger)

  # get logger
  @staticmethod
  def get_logger(name = 'default'):
    return logging.getLogger(name)
  
  def get_my_logger(self, name = 'default'):
    try:
      self.logger.info(f"Logger '{name}' returned")
      return self.my_loggers[name]
    except:
      raise Exception(f"Can't get my_logger with name '{name}'")

  def set_default_my_formatter(self):
    my_formattr = self.my_formatters.get('default')
    if not my_formattr:
      my_formattr = my_formatter.MyFormatter(
        '{asctime} {levelname} ({filename} {lineno}): {message}',
        { 'datefmt': "%Y-%m-%d %H:%M",
          'style': "{"
        }
      )
    return my_formattr

  def set_default_my_handler(self):
    handler = self.my_handlers.get('default')
    if not handler:
      handler = my_stream_handler.MyStreamHandler('default')
      handler.set_formatter(self.my_formatters['default'].formatter)
    return handler

  def set_default_my_logger(self):
    logger = my_logger.MyLogger('default')
    logger.add_handler(self.my_handlers['default'].handler)
    return logger

  def set_my_logger_level(self, level, my_logger = None):
    if not my_logger:
      my_logger = self.get_my_logger()
    my_logger.set_level(level)      






import a1_sys.logging.my_handler as my_handler
from dataclasses import dataclass
import logging

@dataclass
class MyStreamHandler(my_handler.MyHandler) :

  def __init__(self, name):
    super().__init__(name)
    super().set_handler(logging.StreamHandler())

  def get_handler_type(self):
    return 'StreamHandler'


import a1_misc.regex.regex_equal as regex_equal
from dataclasses import dataclass
import logging


@dataclass
class MyLogger:
#class MyLogger(logging.Logger):


  name: str
  handlers: dict[str:]
  logger: logging.Logger
  
  def __init__(self, name, level = None):
    self.name = name
    level = "WARNING" if level is None else level 

    # 
    self.logger = logging.getLogger(name)
    self.logger.level = self.get_logging_level(level)
    #
    self.handlers = {}
  
  def add_handler(self, handler):
    self.logger.addHandler(handler)

  def critical(self, message):
    self.logger.critical(message)

  def debug(self, message):
    self.logger.debug(message)

  def error(self, message):
    self.logger.error(message)

  def get_logger(self):
    return self.logger

  @staticmethod
  #def get_logging_level(self, level):
  def get_logging_level(level):
    match regex_equal.RegexEqual(level):
      case 'DEBUG|debug':
        return logging.DEBUG
      case 'INFO|info':
        return logging.INFO
      case 'WARNING|warning':
        return logging.WARNING
      case 'ERROR|error':
        return logging.ERROR
      case 'CRITICAL|critical':
        return logging.CRITICAL
      case _:
        raise Exception(f"Can't match level '{level}'")

  def info(self, message):
    self.logger.info(message)
  
  def set_level(self, level):
    self.logger.setLevel(self.get_logging_level(level))

  def warning(self, message):
    self.logger.warning(message)




from dataclasses import dataclass

@dataclass
class MyHandler:

  #name: str
  #handler_type: str

  #def __init__(self, name, handler_type):
  def __init__(self, name):    
    self.name = name
    #self.handler_type = handler_type
    self.handler_type = self.get_handler_type()

  def set_formatter(self, formatter):
    self.handler.setFormatter(formatter)
    #pass
  
  def set_handler(self, handler):
    self.handler = handler




import a1_sys.logging.handlers.my_file_handler as my_file_handler
from dataclasses import dataclass
import logging.handlers

@dataclass
class MyRotatingFileHandler(my_file_handler.MyFileHandler) :

  backupCount: int
  maxBytes: int

  def __init__(self, name, filepath, parms = {}):
    # 
    super().__init__(name, filepath, parms)
    #
    self.maxBytes = parms.get('maxBytes', 10000)
    self.backupCount = parms.get('backupCount', 4)
    #
    super().set_handler(
        logging.handlers.RotatingFileHandler(
      self.filepath, mode = self.mode , maxBytes = self.maxBytes, backupCount = self.backupCount, encoding = self.encoding
        )
    )

  def get_handler_type(self):
    return 'RotatingFileHandler'



import a1_sys.logging.my_handler as my_handler
from dataclasses import dataclass
import logging

@dataclass
class MyFileHandler(my_handler.MyHandler) :

  filepath: str
  mode: str
  encoding: str

  def __init__(self, name, filepath, parms = {}):

    self.filepath = filepath
    self.mode = parms.get('mode', 'a')
    self.encoding = parms.get('encoding', 'UTF-8')

    #self.handler_type = self.get_handler_type()

    #super().__init__(name, self.handler_type)
    super().__init__(name)
    super().set_handler(logging.FileHandler(self.filepath, mode = self.mode , encoding = self.encoding))



  def get_handler_type(self):
    return 'FileHandler'



from a1_loggermanager import logger_manager 


### functions

def get_logger_manager():
    return _logger_manager


### initialisation

# set various logger formats
  # message formats
default_format = "{asctime} {levelname} {filename}: {message}"
default_short_format = "{asctime} {levelname}: {message}"
  # date format
date_and_time_format1 = "%Y-%m-%d %H:%M:%S"
time_format1 = "%H:%M:%S"
  # my_formatters parms
myf = (
  { 'name': 'default_format',
    'format': default_format,
    'parms': {
      'datefmt': date_and_time_format1,
      'style': "{"
    }
  }, 
  { 'name': 'default_short_format',
    'format': default_short_format,
    'parms': {
      'datefmt': time_format1,
      'style': "{"
    }
  },
)
  # my_handlers parms
myh = (
  { 'name': 'default_streamhandler',
    'formatter_name': 'default_format',
    'handler_type': 'StreamHandler'
  },
)
  # my_loggers parms
myl = (
  { 'name': 'default_console_logger',
    'level': 'info',
    'handler_names': ('default_streamhandler',),
  },
)

# get a logger manager and add formatter, handler and logger parms
_logger_manager = logger_manager.LoggerManager()
_logger_manager.add_my_formatters(myf)
_logger_manager.add_my_handlers(myh)
_logger_manager.add_my_loggers(myl)




import a1_sys.logging.handlers.my_file_handler as my_file_handler
from dataclasses import dataclass
import datetime
import logging.handlers

@dataclass
class MyTimedRotatingFileHandler(my_file_handler.MyFileHandler) :

  atTime: datetime.time
  backupCount: int
  interval: int
  when: str


  def __init__(self, name, filepath, parms = {}):
    #
    super().__init__(name, filepath, parms)
    # back up 4
    self.atTime = parms.get('atTime', None )  
    self.backupCount = parms.get('backupCount', 4)
    self.interval = parms.get('interval', 4)
    self.when = parms.get('when', 'W0') # (Weekday (0=Monday))
    #
    super().set_handler(
      logging.handlers.TimedRotatingFileHandler(
        self.filepath, 
        when = self.when, 
        interval = self.interval, 
        backupCount = self.backupCount, 
        encoding = self.encoding, 
        atTime = self.atTime
      )
    )

  def get_handler_type(self):
    return 'TimedRotatingFileHandler'



import logging
from dataclasses import dataclass

@dataclass
class MyFormatter:

  format: str
  style: str
  datefmt: str

  def __init__(self, format, parms = {}):
    self.format = format
    self.style = parms.get('style', '{')
    self.datefmt = parms.get('datefmt', '%Y-%m-%d %H:%M')
    self.formatter = logging.Formatter(self.format, style = self.style, datefmt = self.datefmt)

    